/* Copyright 2000 Hiroshi Watabe*/
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "gpetview.h"
gshort lenx,leny;
gfloat maxx,maxy;
GtkCurve *xcurve,*ycurve;
gfloat *promat_x=NULL,*promat_y=NULL;

void roi_profile()
{
  gfloat *promat=NULL;
  gshort x0,y0,x1,y1;
  gfloat deg;
  gint ix,iy;
  gfloat x2,y2;
  //  gshort px[4],py[4];
  gfloat dx1,dx2,dy1,dy2;
  gshort ix1,ix2,iy1,iy2;
  guchar v1,v2,v3,v4;
  gfloat fdat,sum_x;

  if(roi_flag != RECTANGLE){
    return;
  }
  lenx = 2*figure.la;
  leny = 2*figure.sa;
  x0 = -figure.la;
  y0 = -figure.sa;
  deg = figure.degree;
  if(promat==NULL){
    promat = (gfloat *)g_malloc(lenx*leny*sizeof(gfloat));
    promat_x = (gfloat *)g_malloc(lenx*sizeof(gfloat));
    promat_y = (gfloat *)g_malloc(leny*sizeof(gfloat));
  }
  else{
    promat = (gfloat *)g_realloc(promat,lenx*leny*sizeof(gfloat));
    promat_x = (gfloat *)g_realloc(promat_x,lenx*sizeof(gfloat));
    promat_y = (gfloat *)g_realloc(promat_y,leny*sizeof(gfloat));
  }
  if(promat==NULL||promat_x==NULL||promat_y==NULL){
    return; 
  }
  /* we know before affine transformation, x is from -la to +la and y is from -sa to +sa */
  /* so first step by step for one pixel  */
  /* then we apply affine transform to get coordinate on the image */
  for(iy=0;iy<leny;iy++){
    y1 = y0 + iy;
    for(ix=0;ix<lenx;ix++){
      x1 = x0 + ix;
      x2 = x1*cos(deg) + y1*sin(deg) + figure.center.x;
      y2 = -x1*sin(deg) + y1*cos(deg) + figure.center.y;
      ix1 = (short)x2;
      dx1 = x2 - ix1;
      ix2 = ix1 + 1;
      dx2 = 1.0 - dx1;
      iy1 = (short)y2;
      dy1 = y2 - iy1;
      iy2 = iy1 + 1;
      dy2 = 1.0 - dy1;
      /*
	printf("%g %g ix1 %d iy1 %d ix2 %d iy2 %d\n",x2,y2,ix1,ix2,iy1,iy2);
      */
      v1 = pick_point(ix1,iy1);
      v2 = pick_point(ix2,iy1);
      v3 = pick_point(ix2,iy2);
      v4 = pick_point(ix1,iy2);
      fdat = v1*dx2*dy2 + v2*dx1*dy2 + v3*dx1*dy1 + v4*dx2*dy1;
      /*
      printf("%d %d v1 %d %d %d %d dx %g %g %g %g %g\n",ix,iy,v1,v2,v3,v4,dx1,dx2,dy1,dy2,fdat);
      */
      promat[ix + lenx*iy] = fdat;
    }
  }
  /* calculate profile */
  for(iy=0;iy<leny;iy++){
    for(ix=0,sum_x=0;ix<lenx;ix++){
      sum_x += promat[ix + lenx*iy];
    }
    promat_y[iy] = sum_x*img->scalefactor;
  }
  for(ix=0;ix<lenx;ix++){
    for(iy=0,sum_x=0;iy<leny;iy++){
      sum_x += promat[ix + lenx*iy];
    }
    promat_x[ix] = sum_x*img->scalefactor;
  }
  /* calculate average profile */
  for(ix=0,maxx=0;ix<lenx;ix++){
    promat_x[ix]/=(float)leny;
    if(maxx<promat_x[ix]){
      maxx = promat_x[ix];
    }
    /*    printf("pro_x %d %g\n",ix,promat_x[ix]);*/
  }
  for(ix=0,maxy=0;ix<leny;ix++){
    promat_y[ix]/=(float)lenx;
    if(maxy<promat_y[ix]){
      maxy = promat_y[ix];
    }
    /*    printf("pro_y %d %g\n",ix,promat_y[ix]);*/
  }
  xcurve = (GtkCurve *)lookup_widget(profile_window,"profilex_curve");
  ycurve = (GtkCurve *)lookup_widget(profile_window,"profiley_curve");
  gtk_curve_set_range(xcurve,0,lenx,0,maxx);
  gtk_curve_set_range(ycurve,0,leny,0,maxy);
  gtk_curve_set_vector(xcurve,lenx,promat_x);
  gtk_curve_set_vector(ycurve,leny,promat_y);
  gtk_widget_show(profile_window);
  g_free(promat);
}

void on_profilex(GdkEventMotion *event)
{
  gshort width,height;
  gshort posx,posy;
  gfloat ratio_x,ratio_y;
  gchar message[50];

  width = xcurve->graph.widget.allocation.width;
  height = xcurve->graph.widget.allocation.height;
  posx = event->x;
  posy = height - event->y;
  ratio_x = (float)posx/(float)width;
  ratio_y = (float)posy/(float)height;
  ratio_x *= lenx;
  ratio_y *= maxx;
  sprintf(message,"%g %g\n",ratio_x,ratio_y);
  put_string(message,0);
}

void on_profiley(GdkEventMotion *event)
{
  gshort width,height;
  gshort posx,posy;
  gfloat ratio_x,ratio_y;
  gchar message[50];

  width = ycurve->graph.widget.allocation.width;
  height = ycurve->graph.widget.allocation.height;
  posx = event->x;
  posy = height - event->y;
  ratio_x = (float)posx/(float)width;
  ratio_y = (float)posy/(float)height;
  ratio_x *= leny;
  ratio_y *= maxy;
  sprintf(message,"%g %g\n",ratio_x,ratio_y);
  put_string(message,0);
}

void save_profile(FILE *fp)
{
  int i;

  fprintf(fp,"#lenx %d leny %d center_x %d center_y %d degree %g\n",lenx,leny,figure.center.x,figure.center.y,figure.degree);
  fprintf(fp,"#Histgram X\n");
  for(i=0;i<lenx;i++){
    fprintf(fp,"%d %g\n",i,promat_x[i]);
  }
  fprintf(fp,"#Histgram Y\n");
  for(i=0;i<leny;i++){
    fprintf(fp,"%d %g\n",i,promat_y[i]);
  }
  g_free(promat_x);
  g_free(promat_y);
}
